<?php namespace RWMediaIntegration;

class Node {
	
	public static function requestUpdate($node) {
		
		$nid = $node->nid;
		$apiServer = $_SERVER['SERVER_NAME'];
		$apiPathNodeUpdate = $GLOBALS['base_path'] . \RWmedia\settings()['dirs']['api_node_update'];
		$apiUrl = $apiServer . $apiPathNodeUpdate;
		
		$request = [
			'url' => $apiUrl, //node_update api
			'data' => json_encode([
				'apiUrl' => $apiUrl,
				'appId' => \RWmedia\settings()['credentials']['rewired']['appId'],
				'nid' => $nid,
				'ttl' => 50,
			]),
			'requestType' => 'POST',
		];
		
		// return drupal_json_output($request);

		\RWHttp::requestAsync($request);
	}
	
	public static function update() {
		$postRequestData = file_get_contents('php://input');
		$postRequestData = urldecode($postRequestData);
		$dataArr = json_decode($postRequestData, true);
		
		$apiUrl = $dataArr['apiUrl'] ?? '';
		$appId = $dataArr['appId'] ?? '';
		// $idArr = $dataArr['idArr'] ?? []; //TODO delete this line.
		$nid = $dataArr['nid'] ?? '';
		$ttl = $dataArr['ttl'] ?? '';

		// return drupal_json_output($dataArr);

		//answer to HTML GET request
		if ($_SERVER['REQUEST_METHOD'] === 'GET') {
			return drupal_json_output([
				'status-code' => '200',
				'status' => 'Rewired.hu Media API is OK.',
				'request-type' => 'node_update',
			]);
		}

		--$ttl;
		
		$ogImageFound = false;
		
		if (!isset($dataArr['idArr'])) {

			$node = node_load($nid);
			
			// --- SET DEFAULT OG -----------------------------------------------------------
			//create og description
			$ogDescriptionString = \RWMediaIntegration\Database::createOgDescription($node);
			
			//create default og image
			$imageUrl = $_SERVER['SERVER_NAME']. \RWmedia\settings()['dirs']['web_general'] . 'og_image.jpg';
			
			//insert og
			\RWMediaIntegration\Database::insertOg([
				'node' => $node,
				'description' => $ogDescriptionString,
				'imageUrl' => $imageUrl,
			]);
			
			// --- GET IDS -----------------------------------------------------------
			$nodeBodyString = $node->body['und'][0]['value'];
			$urlArr = self::getBbcodeTags($nodeBodyString);
			$dataArr['idArr'] = [];
			if (count($urlArr) > 0) {
				foreach ($urlArr as $url) {
					$dataArr['idArr'][] = \RWMedia\Crypto::rwHash($url);
				}
			}
			
			// --- ANALYZE IDS -----------------------------------------------------------
			self::requestFromBbcode($nodeBodyString);
			
		}
		else {
			// --- GET OG FROM POTENTIAL ENTRIES -----------------------------------------------------------
			//check data in DB
			$entryArr = \RWMedia\Crud::getGroup($dataArr['idArr']);
			
			//filter out Status 7 and 9 entries
			foreach ($entryArr['data'] as $key => $entry) {
				if (
					// $entry['status-code'] === '404' ||
					$entry['status-code'] === '200' && $entry['data']['status'] === '7'
					|| $entry['status-code'] === '200' && $entry['data']['status'] === '9'
				) {
					$idArrPosition = array_search($key, $dataArr['idArr']);
					unset($dataArr['idArr'][$idArrPosition]);
				}
			}
			
			//find the OG image on the first viable entry
			$imageReadPosition = 0;
			foreach ($dataArr['idArr'] as $key => $id) {
				
				$entry = $entryArr['data'][$id];
				
				if ($imageReadPosition === 0 && $entry['status-code'] === '200' && $entry['data']['status'] === '8') {
					
					$backgroundUrl = $entry['data']['background_url'];
					
					if (isset($backgroundUrl) && $backgroundUrl !== '') {
						$ogImageFound = true;
						
						//insert OG image
						$node = node_load($nid);
						\RWMediaIntegration\Database::insertOg([
							'node' => $node,
							'imageUrl' => $backgroundUrl,
						]);
						
						//call any further functions if needed, even from different modules.
						self::afterOgImageFound($node, $backgroundUrl);
						break;
					}
					else {
						//no image at this entry, so filter it from the list, and reset imageReadPosition
						unset($dataArr['idArr'][$key]);
						$imageReadPosition = -1;
					}
				}
				$imageReadPosition++;
			}
		
		}

		//If first entry doesn't have an image, and there is still jobs pending, and there are TTL left
		if (!$ogImageFound && count($dataArr['idArr']) > 0 && $ttl > 0) {
			$request = [
				'url' => $apiUrl, //node_update api
				'data' => json_encode([
					'apiUrl' => $apiUrl,
					'idArr' => $dataArr['idArr'],
					'nid' => $nid,
					'ttl' => $ttl,
					'appId' => $appId,
				]),
				'requestType' => 'POST',
			];
			
			\RWHttp::requestAsync($request, 5);
		}
		
	}

	public static function getBbcodeTags($string) {
		$pattern = '/\[media\](.*?)\[\/media\]/i';
		preg_match_all($pattern, $string, $matches);
		
		return count($matches) ? $matches[1] : [];
	}

	public static function requestFromBbcode($string) {

		$urlArr = self::getBbcodeTags($string);
		
		if (count($urlArr) > 0) {
			$apiServer = $_SERVER['SERVER_NAME']; // ip
			$apiPathSet = $GLOBALS['base_path'] . \RWmedia\settings()['dirs']['api_set'];  // "/rewired/" + "rwmedia/set"
			
			foreach ($urlArr as $url) {
				\RWHttp::requestAsync([
					'url' => "$apiServer$apiPathSet",
					'data' => json_encode([
						'data' => [$url],
						'appId' => \RWmedia\settings()['credentials']['rewired']['appId'],
					]),
					'requestType' => 'POST', //requesting over POST to keep appId safe.
				]);
			}
		}
	}
	
	// Any functions (even from different RW modules), that needs to be called after a viable OG image is found in this Node.
	// Make sure to include module_exists() calls, so individuals modules can be turned off safely, if needed.
	public static function afterOgImageFound($node, $imageUrl) {
		
		if (module_exists('rwnews_and_articles')) {
			\RWNewsAndArticles\ogImageFound($node, $imageUrl);
		}
		
	}

}

?>